# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs
from horizon import views

from horizon.utils import memoized

from adjutant_ui.api import adjutant
from adjutant_ui.content.notifications import tables as notification_tables
from adjutant_ui.content.notifications import tabs as notification_tab


class IndexView(tabs.TabbedTableView):
    tab_group_class = notification_tab.NotificationTabGroup
    template_name = 'notifications/index.html'
    redirect_url = 'horizon:management:notifications:index'
    page_title = _("Admin Notifications")


class NotificationDetailView(views.HorizonTemplateView):
    redirect_url = "horizon:management:notifications:index"
    template_name = 'notifications/detail.html'
    page_title = "Notification: {{ notification.uuid }}"

    def get_context_data(self, **kwargs):
        context = super(NotificationDetailView, self).get_context_data(
            **kwargs)
        notification, task = self.get_data()
        context["notification"] = notification
        context['task'] = task
        context["url"] = reverse(self.redirect_url)
        context["actions"] = self._get_actions(notification=notification)
        return context

    def _get_actions(self, notification):
        table = notification_tables.NotificationTable(self.request)
        return table.render_row_actions(notification)

    @memoized.memoized_method
    def get_data(self):
        try:
            notification = adjutant.notification_obj_get(
                self.request, self.kwargs['notif_id'])
            task = adjutant.task_obj_get(self.request,
                                         task_id=notification.task)
            return notification, task
        except Exception:
            msg = _('Unable to retrieve notification.')
            url = reverse('horizon:management:notifications:index')
            exceptions.handle(self.request, msg, redirect=url)



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import json
import logging
import requests
from urllib.parse import urljoin

from django.conf import settings
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon.utils import functions as utils
from horizon.utils import memoized

from openstack_dashboard.api import base

LOG = logging.getLogger(__name__)
USER = collections.namedtuple('User',
                              ['id', 'name', 'email',
                               'roles', 'cohort', 'status'])
TOKEN = collections.namedtuple('Token',
                               ['action'])

TASK = collections.namedtuple('Task',
                              ['id', 'task_type', 'valid',
                               'request_by', 'request_project',
                               'created_on', 'approved_on', 'page',
                               'completed_on', 'actions', 'status'])

NOTIFICATION = collections.namedtuple('Notification',
                                      ['uuid', 'notes', 'error', 'created_on',
                                       'acknowledged', 'task'])

QUOTA_SIZE = collections.namedtuple('QuotaSize',
                                    ['id', 'name', 'cinder',
                                     'nova', 'neutron'])

REGION_QUOTA = collections.namedtuple('RegionQuota',
                                      ['id', 'region',
                                       'quota_size', 'preapproved_quotas'])

REGION_QUOTA_VALUE = collections.namedtuple('RegionQuotaValue',
                                            ['id', 'name',
                                             'service', 'current_quota',
                                             'current_usage', 'percent',
                                             'size_blob', 'important'])

SIZE_QUOTA_VALUE = collections.namedtuple('SizeQuotaValue',
                                          ['id', 'name', 'service',
                                           'value', 'current_quota',
                                           'current_usage', 'percent'])

QUOTA_TASK = collections.namedtuple(
    'QuotaTask',
    ['id', 'regions', 'size', 'user', 'created', 'valid', 'status'])


# NOTE(amelia): A list of quota names that we consider to be the most
# relevant to customers to be shown initially on the update page.
# These can be overriden in the local_settings file:
# IMPORTANT_QUOTAS = {<service>: [<quota_name>], }
IMPORTANT_QUOTAS = {
    'nova': [
        'instances', 'cores', 'ram',
    ],
    'cinder': [
        'volumes', 'snapshots', 'gigabytes',
    ],
    'neutron': [
        'network', 'floatingip', 'router', 'security_group',
    ],
    'octavia': [
        'load_balancer',
    ],
}


# NOTE(adriant): Quotas that should be hidden by default.
# Can be overriden in the local_settings file by setting:
# HIDDEN_QUOTAS = {<service>: [<quota_name>], }
# or disabled entirely with: HIDDEN_QUOTAS = {}
HIDDEN_QUOTAS = {
    # these values have long since been deprecated from Nova
    'nova': [
        'security_groups', 'security_group_rules',
        'floating_ips', 'fixed_ips',
    ],
    # these by default have no limit
    'cinder': [
        'per_volume_gigabytes', 'volumes_lvmdriver-1',
        'gigabytes_lvmdriver-1', 'snapshots_lvmdriver-1',

    ],
    'neutron': [
        'subnetpool',
    ],
}


ROLE_TRANSLATIONS = {
    'project_admin': _('Project Administrator'),
    'project_mod': _('Project Moderator'),
    '_member_': _('Project Member'),
    'Member': _('Project Member'),
    'heat_stack_owner': _('Heat Stack Owner'),
    'project_readonly': _('Project Read-only'),
    'compute_start_stop': _('Compute Start/Stop'),
    'object_storage': _('Object Storage')
}


def get_role_text(name):
    # Gets the role text for a given role.
    # If it doesn't exist will simply return the role name.
    role_translations = getattr(settings, 'ROLE_TRANSLATIONS', None)
    if role_translations is None:
        role_translations = ROLE_TRANSLATIONS
    if name in role_translations:
        return role_translations[name].format()
    return name


SERVICE_TRANSLATIONS = {
    'cinder': _('Volume'),
    'neutron': _('Networking'),
    'nova': _('Compute'),
    'octavia': _('Load Balancer'),
}


def get_service_type(name):
    # Takes service names and returns a 'nice' name of where they
    # are from
    service_translations = getattr(settings, 'SERVICE_TRANSLATIONS', None)
    if service_translations is None:
        service_translations = SERVICE_TRANSLATIONS
    if name in service_translations:
        return service_translations[name].format()
    return name


class AdjutantApiError(BaseException):
    pass


def _get_endpoint_url(request):
    # If the request is made by an anonymous user, this endpoint request fails.
    # Thus, we must hardcode this in Horizon.
    if getattr(request.user, "service_catalog", None):
        try:
            url = base.url_for(request, service_type='admin-logic')
        except exceptions.ServiceCatalogException:
            url = base.url_for(request, service_type='registration')
    else:
        try:
            url = getattr(settings, 'OPENSTACK_ADJUTANT_URL')
        except AttributeError:
            url = getattr(settings, 'OPENSTACK_REGISTRATION_URL')

    # Ensure ends in slash
    if not url.endswith('/'):
        url += '/'

    return url


def _request(request, method, url, headers, **kwargs):
    try:
        endpoint_url = _get_endpoint_url(request)
        url = urljoin(endpoint_url, url)
        session = requests.Session()
        data = kwargs.pop("data", None)
        return session.request(method, url, headers=headers,
                               data=data, **kwargs)
    except Exception as e:
        LOG.error(e)
        raise


def head(request, url, **kwargs):
    return _request(request, 'HEAD', url, **kwargs)


def get(request, url, **kwargs):
    return _request(request, 'GET', url, **kwargs)


def post(request, url, **kwargs):
    return _request(request, 'POST', url, **kwargs)


def put(request, url, **kwargs):
    return _request(request, 'PUT', url, **kwargs)


def patch(request, url, **kwargs):
    return _request(request, 'PATCH', url, **kwargs)


def delete(request, url, **kwargs):
    return _request(request, 'DELETE', url, **kwargs)


def user_invite(request, user):
    headers = {'Content-Type': 'application/json',
               'X-Auth-Token': request.user.token.id}
    user['project_id'] = request.user.tenant_id
    return post(request, 'openstack/users',
                headers=headers, data=json.dumps(user))


def user_list(request):
    users = []
    try:
        headers = {'Content-Type': 'application/json',
                   'X-Auth-Token': request.user.token.id}
        resp = json.loads(get(request, 'openstack/users',
                              headers=headers).content)

        for user in resp['users']:
            users.append(
                USER(
                    id=user['id'],
                    name=user['name'],
                    email=user['email'],
                    roles=user['roles'],
                    status=user['status'],
                    cohort=user['cohort']
                )
            )
    except Exception as e:
        LOG.error(e)
        raise
    return users


def user_get(request, user_id):
    try:
        headers = {'X-Auth-Token': request.user.token.id}
        resp = get(request, 'openstack/users/%s' % user_id,
                   headers=headers).content
        return json.loads(resp)
    except Exception as e:
        LOG.error(e)
        raise


def user_roles_update(request, user):
    try:
        headers = {'Content-Type': 'application/json',
                   'X-Auth-Token': request.user.token.id}
        user['project_id'] = request.user.tenant_id
        user['roles'] = user.roles
        return put(request, 'openstack/users/%s/roles' % user['id'],
                   headers=headers,
                   data=json.dumps(user))
    except Exception as e:
        LOG.error(e)
        raise


def user_roles_add(request, user_id, roles):
    try:
        headers = {'Content-Type': 'application/json',
                   'X-Auth-Token': request.user.token.id}
        params = {}
        params['project_id'] = request.user.tenant_id
        params['roles'] = roles
        return put(request, 'openstack/users/%s/roles' % user_id,
                   headers=headers,
                   data=json.dumps(params))
    except Exception as e:
        LOG.error(e)
        raise


def user_roles_remove(request, user_id, roles):
    try:
        headers = {'Content-Type': 'application/json',
                   'X-Auth-Token': request.user.token.id}
        params = {}
        params['project_id'] = request.user.tenant_id
        params['roles'] = roles
        return delete(request, 'openstack/users/%s/roles' % user_id,
                      headers=headers,
                      data=json.dumps(params))
    except Exception as e:
        LOG.error(e)
        raise


def user_revoke(request, user_id):
    try:
        headers = {'Content-Type': 'application/json',
                   'X-Auth-Token': request.user.token.id}
        data = dict()
        return delete(request, 'openstack/users/%s' % user_id,
                      headers=headers,
                      data=json.dumps(data))
    except Exception as e:
        LOG.error(e)
        raise


def user_invitation_resend(request, user_id):
    headers = {'Content-Type': 'application/json',
               'X-Auth-Token': request.user.token.id}
    # For non-active users, the task id is the same as their userid
    # For active users, re-sending an invitation doesn't make sense.
    data = {
        "task": user_id
    }
    return post(request, 'tokens',
                headers=headers,
                data=json.dumps(data))


def valid_roles_get(request):
    headers = {'Content-Type': 'application/json',
               'X-Auth-Token': request.user.token.id}
    role_data = get(request, 'openstack/roles', headers=headers)
    return role_data.json()


def valid_role_names_get(request):
    roles_data = valid_roles_get(request)
    role_names = [r['name'] for r in roles_data['roles']]
    return role_names


def token_get(request, token, data):
    headers = {'Content-Type': 'application/json'}
    return get(request, 'tokens/%s' % token,
               data=json.dumps(data), headers=headers)


def token_submit(request, token, data):
    headers = {"Content-Type": "application/json"}
    return post(request, 'tokens/%s' % token,
                data=json.dumps(data), headers=headers)


def token_reissue(request, task_id):
    headers = {'Content-Type': 'application/json',
               'X-Auth-Token': request.user.token.id}
    data = {'task': task_id}
    return post(request, 'tokens/',
                data=json.dumps(data), headers=headers)


def email_update(request, email):
    headers = {'Content-Type': 'application/json',
               'X-Auth-Token': request.user.token.id}
    data = {
        'new_email': email
    }
    return post(request, 'openstack/users/email-update',
                data=json.dumps(data), headers=headers)


def forgotpassword_submit(request, data):
    headers = {"Content-Type": "application/json"}
    try:
        return post(request, 'openstack/users/password-reset',
                    data=json.dumps(data),
                    headers=headers)
    except Exception as e:
        LOG.error(e)
        raise


def signup_submit(request, data):
    headers = {"Content-Type": "application/json"}
    try:
        return post(request, 'openstack/sign-up',
                    data=json.dumps(data),
                    headers=headers)
    except Exception as e:
        LOG.error(e)
        raise


def notification_list(request, filters={}, page=1):
    notifs_per_page = utils.get_page_size(request)
    headers = {"Content-Type": "application/json",
               'X-Auth-Token': request.user.token.id}

    response = get(request, 'notifications', headers=headers,
                   params={'filters': json.dumps(filters), 'page': page,
                           'notifications_per_page': notifs_per_page})
    if not response.status_code == 200:
        if response.json() == {'error': 'Empty page'}:
            raise AdjutantApiError("Empty Page")
        raise BaseException

    notificationlist = []
    for notification in response.json()['notifications']:
        notificationlist.append(notification_obj_get(
            request, notification=notification))
    has_more = response.json()['has_more']
    has_prev = response.json()['has_prev']
    return notificationlist, has_prev, has_more


def notification_get(request, uuid):
    headers = {"Content-Type": "application/json",
               'X-Auth-Token': request.user.token.id}

    response = get(request, 'notifications/%s/' % uuid, headers=headers)
    return response


def notification_obj_get(request, notification_id=None, notification=None):
    if not notification:
        notification = notification_get(request, notification_id).json()

    if notification['error']:
        notes = notification['notes'].get('errors')
    else:
        notes = notification['notes'].get('notes')

    if not notes:
        notes = notification['notes']
    if isinstance(notes, list) and len(notes) == 1:
        notes = notes[0]

    if not isinstance(notes, str):
        notes = json.dumps(notes)

    return NOTIFICATION(uuid=notification['uuid'],
                        task=notification['task'],
                        error=notification['error'],
                        created_on=notification['created_on'],
                        acknowledged=notification['acknowledged'],
                        notes=notes)


def notifications_acknowlege(request, notification_id=None):
    headers = {"Content-Type": "application/json",
               'X-Auth-Token': request.user.token.id}
    # Takes either a single notification id or a list of them
    # and acknowleges all of them
    if isinstance(notification_id, list):
        data = {'notifications': notification_id}
        return post(request, 'notifications', data=json.dumps(data),
                    headers=headers)
    else:
        url = "notifications/%s/" % notification_id
        return post(request, url, data=json.dumps({'acknowledged': True}),
                    headers=headers)


def task_list(request, filters={}, page=1):
    tasks_per_page = utils.get_page_size(request)
    tasklist = []
    prev = more = False
    try:
        headers = {"Content-Type": "application/json",
                   'X-Auth-Token': request.user.token.id}
        params = {
            "filters": json.dumps(filters),
            "page": page,
            "tasks_per_page": tasks_per_page
        }
        resp = get(request, "tasks", params=params, data=json.dumps({}),
                   headers=headers).json()
        prev = resp['has_prev']
        more = resp['has_more']
        for task in resp['tasks']:
            tasklist.append(task_obj_get(request, task=task, page=page))
        return tasklist, prev, more
    except Exception as e:
        LOG.error(e)
        raise


def task_get(request, task_id):
    # Get a single task
    headers = {"Content-Type": "application/json",
               'X-Auth-Token': request.user.token.id}

    return get(request, "tasks/%s" % task_id,
               headers=headers)


def task_obj_get(request, task_id=None, task=None, page=0):
    if not task:
        task = task_get(request, task_id).json()

    status = "Awaiting Approval"
    if task['cancelled']:
        status = "Cancelled"
    elif task['completed_on']:
        status = "Completed"
    elif task['approved_on']:
        status = "Approved; Incomplete"

    valid = False not in [action['valid'] for
                          action in task['actions']]
    return TASK(
        id=task['uuid'],
        task_type=task['task_type'],
        valid=valid,
        request_by=task['keystone_user'].get('username', '-'),
        request_project=task['keystone_user'].get('project_name', '-'),
        status=status,
        created_on=task['created_on'],
        approved_on=task['approved_on'],
        completed_on=task['completed_on'],
        actions=task['actions'],
        page=page
    )


def task_cancel(request, task_id):
    headers = {"Content-Type": "application/json",
               'X-Auth-Token': request.user.token.id}

    return delete(request, "tasks/%s" % task_id,
                  headers=headers)


def task_approve(request, task_id):
    headers = {"Content-Type": "application/json",
               'X-Auth-Token': request.user.token.id}

    return post(request, "tasks/%s" % task_id,
                data=json.dumps({"approved": True}), headers=headers)


def task_update(request, task_id, new_data):
    headers = {"Content-Type": "application/json",
               'X-Auth-Token': request.user.token.id}

    return put(request, "tasks/%s" % task_id,
               data=new_data, headers=headers)


def task_revalidate(request, task_id):
    task = task_get(request, task_id=task_id).json()

    data = {}
    for action_data in [action['data'] for action in task['actions']]:
        data.update(action_data)

    return task_update(request, task_id, json.dumps(data))


# Quota management functions
def _is_quota_hidden(service, resource):
    hidden_quotas = getattr(settings, 'HIDDEN_QUOTAS', None)
    if hidden_quotas is None:
        hidden_quotas = HIDDEN_QUOTAS
    return service in hidden_quotas and resource in hidden_quotas[service]


def _is_quota_important(service, resource):
    important_quotas = getattr(settings, 'IMPORTANT_QUOTAS', None)
    if important_quotas is None:
        important_quotas = IMPORTANT_QUOTAS
    return (
        service in important_quotas and resource in important_quotas[service])


@memoized.memoized_method
def _get_quota_information(request, regions=None, include_usage=True):
    headers = {'Content-Type': 'application/json',
               'X-Auth-Token': request.user.token.id}
    params = {'include_usage': include_usage}
    if regions:
        params['regions'] = regions
    try:
        return get(request, 'openstack/quotas/',
                   params=params, headers=headers).json()
    except Exception as e:
        LOG.error(e)
        raise


def quota_sizes_get(request, region=None):
    # Gets the list of quota sizes, and a json blob defining what they
    # have for each of the services
    # Region param is useless here, but nedded for memoized decorator to work
    quota_sizes_dict = {}

    resp = _get_quota_information(request, regions=region, include_usage=False)

    for size_name, size in resp['quota_sizes'].items():
        quota_sizes_dict[size_name] = QUOTA_SIZE(
            id=size_name,
            name=size_name,
            cinder=json.dumps(size['cinder'], indent=1),
            nova=json.dumps(size['nova'], indent=1),
            neutron=json.dumps(size['neutron'], indent=1),
        )

    quota_sizes = []
    for size in resp['quota_size_order']:
        quota_sizes.append(quota_sizes_dict[size])

    return quota_sizes


def size_details_get(request, size, region=None):
    """Gets the current details of the size as well as the current region's
    quota
    """
    quota_details = []

    if not region:
        region = request.user.services_region
    resp = _get_quota_information(request, regions=region)

    data = resp['quota_sizes'][size]
    try:
        region = resp['regions'][0]
    except IndexError:
        region = {}
    current_quota = region.get('current_quota', {})
    current_usage = region.get('current_usage', {})
    for service, values in data.items():
        for resource, value in values.items():
            if _is_quota_hidden(service, resource):
                continue

            quota = current_quota.get(service, {}).get(resource, '-')
            usage = current_usage.get(service, {}).get(resource, '-')
            percent = (
                '-'
                if not value or usage == '-'
                else (usage / value)
            )

            quota_details.append(
                SIZE_QUOTA_VALUE(
                    id=resource,
                    name=resource,
                    service=service,
                    value=value,
                    current_quota=quota,
                    current_usage=usage,
                    percent=percent
                )
            )
    return quota_details


def quota_details_get(request, region):
    quota_details = []

    resp = _get_quota_information(request, regions=region)

    try:
        region = resp['regions'][0]
    except IndexError:
        region = {}
    data = region.get('current_quota', {})

    for service, values in data.items():
        for name, value in values.items():
            if _is_quota_hidden(service, name):
                continue

            try:
                if value < 0:
                    value = 'No Limit'
            except (TypeError):
                pass
            usage = resp['regions'][0]['current_usage'][service].get(name)
            try:
                percent = float(usage) / value
            except (TypeError, ZeroDivisionError):
                percent = '-'

            size_blob = {}
            for size_name, size_data in resp['quota_sizes'].items():
                size_blob[size_name] = size_data[service].get(name, '-')

            if name != 'id':
                quota_details.append(
                    REGION_QUOTA_VALUE(
                        id=name,
                        name=name,
                        service=service,
                        current_quota=value,
                        current_usage=usage,
                        percent=percent,
                        size_blob=size_blob,
                        important=_is_quota_important(service, name)
                    )
                )
    return quota_details


def region_quotas_get(request, region=None):
    quota_details = []

    resp = _get_quota_information(request, regions=region, include_usage=False)

    data = resp['regions']
    for region_values in data:
        quota_details.append(
            REGION_QUOTA(
                id=region_values['region'],
                region=region_values['region'],
                quota_size=region_values['current_quota_size'],
                preapproved_quotas=', '.join(region_values[
                    'quota_change_options'])
            )
        )
    return quota_details


def quota_tasks_get(request, region=None):
    # Region param only used to help with memoized decorator
    quota_tasks = []

    resp = _get_quota_information(request, regions=region, include_usage=False)

    for task in resp['active_quota_tasks']:
        quota_tasks.append(
            QUOTA_TASK(
                id=task['id'],
                regions=', '.join(task['regions']),
                size=task['size'],
                user=task['request_user'],
                created=task['task_created'].split("T")[0],
                valid=task['valid'],
                status=task['status'],
            )
        )
    return quota_tasks


def update_quotas(request, size, regions=[]):
    headers = {'Content-Type': 'application/json',
               'X-Auth-Token': request.user.token.id}
    data = {
        'size': size,
    }
    if regions:
        data['regions'] = regions

    return post(request, 'openstack/quotas/',
                data=json.dumps(data),
                headers=headers)



# Copyright (c) 2016 Catalyst IT Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tables as horizon_tables

from adjutant_ui.api import adjutant
from adjutant_ui.content.quota import forms as quota_forms
from adjutant_ui.content.quota import tables as quota_tables


class IndexView(horizon_tables.MultiTableView):
    page_title = _("Quota Management")
    table_classes = (quota_tables.RegionOverviewTable,
                     quota_tables.SizeOverviewTable,
                     quota_tables.QuotaTasksTable)
    template_name = 'management/quota/index.html'

    def get_region_overview_data(self):
        try:
            return adjutant.region_quotas_get(self.request)
        except Exception:
            exceptions.handle(self.request, _('Failed to list quota sizes.'))
            return []

    def get_size_overview_data(self):
        try:
            return adjutant.quota_sizes_get(self.request)
        except Exception:
            exceptions.handle(self.request, _('Failed to list quota sizes.'))
            return []

    def get_quota_tasks_data(self):
        try:
            return adjutant.quota_tasks_get(self.request)
        except Exception:
            exceptions.handle(self.request, _('Failed to list quota tasks.'))
            return []


class RegionDetailView(horizon_tables.DataTableView):
    table_class = quota_tables.RegionQuotaDetailTable
    template_name = 'management/quota/region_detail.html'
    page_title = _("{{ region }} Quota Details")

    def get_data(self):
        try:
            return adjutant.quota_details_get(self.request,
                                              self.kwargs['region'])
        except Exception:
            exceptions.handle(self.request, _('Failed to list quota sizes.'))
            return []

    def get_context_data(self, **kwargs):
        context = super(RegionDetailView, self).get_context_data(**kwargs)
        context['region'] = self.kwargs['region']
        return context


class QuotaSizeView(horizon_tables.DataTableView):
    table_class = quota_tables.QuotaDetailUsageTable
    template_name = 'management/quota/size_detail.html'
    page_title = _("{{ size | title }} Quota Details")

    def get_data(self):
        try:
            return adjutant.size_details_get(self.request,
                                             size=self.kwargs['size'])
        except Exception:
            exceptions.handle(self.request, _('Failed to list quota size.'))
            return []

    def get_context_data(self, **kwargs):
        # request.user.services_region
        context = super(QuotaSizeView, self).get_context_data(**kwargs)
        context['size'] = self.kwargs['size']
        # NOTE(callumdickinson): Necessary because the title filter
        # is overloaded in the title block.
        context['size_titlecase'] = self.kwargs['size'].title()
        return context


class RegionUpdateView(forms.ModalFormView, horizon_tables.MultiTableView):
    form_class = quota_forms.UpdateQuotaForm
    table_classes = (quota_tables.ChangeSizeDisplayTable, )
    submit_url = 'horizon:management:quota:update'
    context_object_name = 'ticket'
    template_name = 'management/quota/update.html'
    success_url = reverse_lazy("horizon:management:quota:index")
    page_title = _("Update Quota")

    def get_change_size_data(self):
        try:
            return adjutant.quota_details_get(self.request,
                                              region=self.kwargs['region'])
        except Exception:
            exceptions.handle(self.request, _('Failed to list quota sizes.'))
            return []

    def get_object(self):
        return adjutant.region_quotas_get(self.request,
                                          region=self.kwargs['region'])[0]

    def get_context_data(self, **kwargs):
        context = super(RegionUpdateView, self).get_context_data(**kwargs)
        context['region'] = self.get_object()
        args = (self.kwargs['region'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        context['form'] = self.get_form()
        return context

    def get_form_kwargs(self):
        kwargs = super(RegionUpdateView, self).get_form_kwargs()
        sizes = adjutant.quota_sizes_get(
            self.request, region=self.kwargs['region'])
        kwargs['size_choices'] = []

        region = self.get_object()
        for size in sizes:
            if region.quota_size == size.name:
                continue
            if size.name not in region.preapproved_quotas:
                kwargs['size_choices'].append(
                    [size.id, "%s (requires approval)" % size.name.title()])
            else:
                kwargs['size_choices'].append([size.id, size.name.title()])
        return kwargs

    def get_initial(self):
        region = self.get_object()
        data = {'id': region.id,
                'region': region.region,
                'quota_size': region.quota_size,
                'preapproved_quotas': region.preapproved_quotas
                }
        return data

    def post(self, request, *args, **kwargs):
        # NOTE(amelia): The multitableview overides the form view post
        #               this reinstates it.
        form = self.get_form()
        if form.is_valid():
            return self.form_valid(form)
        else:
            return self.form_invalid(form)



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from django.conf import settings
from django import http
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon.utils import functions as utils

from adjutant_ui.api import adjutant
from adjutant_ui.content.token import forms as token_forms


def _logout_msg_response(request, msg):
    response = http.HttpResponseRedirect(settings.LOGOUT_URL)
    utils.add_logout_reason(request, response, msg)
    return response


def _logout_msg_response_success(request, msg):
    response = _logout_msg_response(request, msg)
    response.set_cookie('logout_status', 'success', max_age=10)
    return response


def submit_token_router(request, *args, **kwargs):
    """Routes requests to the correct view, based on submission parameters."""

    # Get details of the token
    token_uuid = kwargs['token']
    token = adjutant.token_get(request, token_uuid, {})
    if not token or token.status_code != 200:
        msg = _("Invalid token. Please request another.")
        return _logout_msg_response(request, msg)

    json = token.json()
    if 'password' in json['required_fields']:
        return SubmitTokenPasswordView.as_view()(request, *args, **kwargs)
    elif 'confirm' in json['required_fields']:

        if 'UpdateUserEmailAction' in json['actions']:
            return UpdateEmailTokenSubmitView.as_view()(
                request, *args, **kwargs)
        return SubmitTokenConfirmView.as_view()(request, *args, **kwargs)

    return _logout_msg_response(request, _("Unsupported token type."))


class SubmitTokenPasswordView(forms.ModalFormView):
    form_class = token_forms.PasswordForm
    template_name = 'token/setpassword.html'

    def get(self, request, *args, **kwargs):
        sc = super(SubmitTokenPasswordView, self)
        return sc.get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        sc = super(SubmitTokenPasswordView, self)
        return sc.post(request, *args, **kwargs)

    def form_valid(self, form):
        # All form data has been validated and POSTed.
        # We can now submit the token for processing along with parameters.
        # All codepaths here return a Redirect response.
        parameters = {
            'password': form.cleaned_data['new_password']
        }
        token_uuid = self.kwargs['token']
        token_response = adjutant.token_submit(form.request,
                                               token_uuid,
                                               parameters)

        if token_response.ok:
            msg = _("Password successfully set. Please log in to continue.")
            return _logout_msg_response_success(form.request, msg)

        msg = (_("Token form submission failed. Response code %(code)s.") %
               {'code': token_response.status_code})
        return _logout_msg_response(form.request, msg)

    def get_context_data(self, **kwargs):
        sc = super(SubmitTokenPasswordView, self)
        context = sc.get_context_data(**kwargs)
        try:
            context['token'] = self.kwargs['token']
        except Exception:
            exceptions.handle(self.request)
        return context


class SubmitTokenConfirmView(forms.ModalFormView):
    form_class = token_forms.ConfirmForm
    template_name = 'token/tokenconfirm.html'
    success_msg = _("Welcome to the project! Please log in to continue.")

    def get(self, request, *args, **kwargs):
        sc = super(SubmitTokenConfirmView, self)
        return sc.get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        sc = super(SubmitTokenConfirmView, self)
        return sc.post(request, *args, **kwargs)

    def form_valid(self, form):
        token_uuid = self.kwargs['token']
        parameters = {
            'confirm': True
        }
        token_response = adjutant.token_submit(form.request,
                                               token_uuid,
                                               parameters)

        if token_response.ok:
            return _logout_msg_response_success(form.request, self.success_msg)

        msg = (_("Token form submission failed. "
                 "Response code %(code)s.") % {'code':
                                               token_response.status_code})
        return _logout_msg_response(form.request, msg)

    def get_context_data(self, **kwargs):
        sc = super(SubmitTokenConfirmView, self)
        context = sc.get_context_data(**kwargs)
        try:
            context['token'] = self.kwargs['token']
        except Exception:
            exceptions.handle(self.request)
        return context


class UpdateEmailTokenSubmitView(SubmitTokenConfirmView):
    template_name = 'token/emailtokenconfirm.html'
    success_msg = _("Your email has been updated! Please log in to continue.")



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Default to Horizons test settings to avoid any missing keys
from horizon.test.settings import *  # noqa
from openstack_dashboard.test.settings import *  # noqa

# Update the dashboards with adjutant_ui
import openstack_dashboard.enabled
from openstack_dashboard.utils import settings

import adjutant_ui.enabled

# pop these keys to avoid log warnings about deprecation
# update_dashboards will populate them anyway
HORIZON_CONFIG.pop('dashboards', None)
HORIZON_CONFIG.pop('default_dashboard', None)

settings.update_dashboards(
    [
        adjutant_ui.enabled,
        openstack_dashboard.enabled,
    ],
    HORIZON_CONFIG,
    INSTALLED_APPS
)

# Ensure any duplicate apps are removed after the update_dashboards call
INSTALLED_APPS = list(set(INSTALLED_APPS))



# The slug of the panel to be added to HORIZON_CONFIG. Required.
PANEL = 'quota'
# The slug of the dashboard the PANEL associated with. Required.
PANEL_DASHBOARD = 'management'

PANEL_GROUP = 'default'

# Python panel class of the PANEL to be added.
ADD_PANEL = 'adjutant_ui.content.quota.panel.QuotaPanel'



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tabs

from horizon.utils import memoized

from adjutant_ui.api import adjutant
from adjutant_ui.content.tasks import forms as task_forms
from adjutant_ui.content.tasks import tables as task_tables
from adjutant_ui.content.tasks import tabs as task_tabs

import json


class IndexView(tabs.TabbedTableView):
    tab_group_class = task_tabs.TaskTabs
    template_name = 'management/tasks/index.html'
    redirect_url = 'horizon:management:tasks:index'
    page_title = _("Admin Tasks")


class TaskDetailView(tabs.TabView):
    tab_group_class = task_tabs.TaskDetailTabs
    template_name = 'horizon/common/_detail.html'
    redirect_url = 'horizon:management:tasks:index'
    page_title = "{{ task.task_type }}"

    def get_context_data(self, **kwargs):
        context = super(TaskDetailView, self).get_context_data(**kwargs)
        task = self.get_data()
        context["task"] = task
        context["url"] = reverse(self.redirect_url)
        context["actions"] = self._get_actions(
            adjutant.task_obj_get(self.request, task=task))
        return context

    def _get_actions(self, task):
        if task.status == 'Approved; Incomplete':
            table = task_tables.ApprovedTaskTable(self.request)
        else:
            table = task_tables.TaskTable(self.request)
        return table.render_row_actions(task)

    @memoized.memoized_method
    def get_data(self):
        return adjutant.task_get(self.request, self.kwargs['task_id']).json()

    def get_tabs(self, request, *args, **kwargs):
        task = self.get_data()
        return self.tab_group_class(request, task=task, **kwargs)


class UpdateTaskView(forms.ModalFormView):
    form_class = task_forms.UpdateTaskForm
    form_id = "update_user_form"
    modal_header = _("Update Task")
    submit_label = _("Update")
    submit_url = 'horizon:management:tasks:update'
    template_name = 'management/tasks/update.html'
    context_object_name = 'project_users'
    success_url = "horizon:management:tasks:detail"
    page_title = _("Update Task")

    def get_success_url(self):
        return reverse(self.success_url,
                       args=(self.kwargs['task_id'],))

    @memoized.memoized_method
    def get_object(self):
        try:
            return adjutant.task_get(self.request,
                                     self.kwargs['task_id']).json()
        except Exception:
            msg = _('Unable to retrieve user.')
            url = reverse('horizon:management:tasks:index')
            exceptions.handle(self.request, msg, redirect=url)

    def get_context_data(self, **kwargs):
        context = super(UpdateTaskView, self).get_context_data(**kwargs)
        context['task'] = self.get_object()
        args = (self.kwargs['task_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        task = self.get_object()
        task_data = {}
        for data in [action['data'] for action in task['actions']]:
            task_data.update(data)
        data = {'task_id': self.kwargs['task_id'],
                'task_type': task['task_type'],
                'task_data': json.dumps(task_data, indent=4),
                }
        return data



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tables
from horizon.utils import memoized

from adjutant_ui.content.project_users import tables as users_tables

from adjutant_ui.content.project_users import forms as users_forms

from adjutant_ui.api import adjutant


class InviteUserView(forms.ModalFormView):
    form_class = users_forms.InviteUserForm
    form_id = "invite_user_form"
    modal_header = _("Invite User")
    submit_label = _("Invite User")
    submit_url = reverse_lazy('horizon:management:project_users:invite')
    template_name = 'management/project_users/invite.html'
    context_object_name = 'project_users'
    success_url = reverse_lazy("horizon:management:project_users:index")
    page_title = _("Invite User")


class UpdateUserView(forms.ModalFormView):
    form_class = users_forms.UpdateUserForm
    form_id = "update_user_form"
    modal_header = _("Update User")
    submit_label = _("Update User")
    submit_url = 'horizon:management:project_users:update'
    template_name = 'management/project_users/update.html'
    context_object_name = 'project_users'
    success_url = reverse_lazy("horizon:management:project_users:index")
    page_title = _("Update User")

    @memoized.memoized_method
    def get_object(self):
        try:
            return adjutant.user_get(self.request,
                                     self.kwargs['user_id'])
        except Exception:
            msg = _('Unable to retrieve user.')
            url = reverse('horizon:management:project_users:index')
            exceptions.handle(self.request, msg, redirect=url)

    def get_context_data(self, **kwargs):
        context = super(UpdateUserView, self).get_context_data(**kwargs)
        context['user'] = self.get_object()
        args = (self.kwargs['user_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        user = self.get_object()
        data = {'id': self.kwargs['user_id'],
                'name': user['username'],
                'roles': user['roles'],
                }
        return data


class UsersView(tables.DataTableView):
    table_class = users_tables.UsersTable
    template_name = 'management/project_users/index.html'
    page_title = _("Project Users")

    def get_data(self):
        try:
            return adjutant.user_list(self.request)
        except Exception:
            exceptions.handle(self.request, _('Failed to list users.'))
            return []



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from openstack_dashboard.test.test_data import utils

from adjutant_ui.test import test_data

TEST = utils.TestData(test_data.data)


def mock_resource(resource):
    """Utility function to make mocking more DRY"""

    mocked_data = \
        [mock.Mock(**{'to_dict.return_value': item}) for item in resource]

    return mocked_data



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import exceptions
from horizon import tables

from adjutant_ui.api import adjutant


class AcknowlegeNotifcation(tables.BatchAction):
    name = 'acknowlege'
    help_text = _("This will acknowlege all selected tasks.")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Acknowlege Notification",
            u"Acknowlege Notifications",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Acknowleged Notification",
            u"Acknowleged Notifications",
            count
        )

    def action(self, request, obj_id):
        result = adjutant.notifications_acknowlege(request, obj_id)
        if not result or result.status_code not in [200, 202]:
            exception = exceptions.NotAvailable()
            exception._safe_message = False
            raise exception

    def allowed(self, request, notification=None):
        if notification:
            return not(notification.acknowledged)
        return True


def get_task_link(datum):
    return reverse("horizon:management:tasks:detail",
                   args=(datum.task,))


class ErrorRow(tables.Row):

    def __init__(self, *args, **kwargs):
        super(ErrorRow, self).__init__(*args, **kwargs)
        if not self.datum:
            return
        if self.datum.error:
            self.classes.append('danger')


class NotificationTable(tables.DataTable):
    uuid = tables.Column('uuid', verbose_name=_('Notification ID'),
                         link="horizon:management:notifications:detail")
    task = tables.Column('task', verbose_name=_('Task ID'),
                         link=get_task_link)
    error = tables.Column('error', verbose_name=_('Error'))
    created_on = tables.Column('created_on',
                               verbose_name=_('Created On'))
    notes = tables.Column('notes')

    class Meta(object):
        template = 'notifications/table_override.html'
        name = 'notification_table'
        verbose_name = _('Unacknowledged Notifications')
        table_actions = (AcknowlegeNotifcation, )
        row_actions = (AcknowlegeNotifcation, )
        row_class = ErrorRow
        prev_pagination_param = pagination_param = 'task_page'

    def get_prev_marker(self):
        return str(int(self.page) - 1) if self.data else ''

    def get_marker(self):
        return str(int(self.page) + 1) if self.data else ''

    def get_object_display(self, obj):
        return obj.uuid

    def get_object_id(self, obj):
        return obj.uuid


class AcknowlegedNotificationTable(NotificationTable):

    class Meta(object):
        name = 'acknowleged_table'
        verbose_name = _('Acknowleged Notifications')
        prev_pagination_param = pagination_param = 'acknowledged_page'
        table_actions = ()



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.shortcuts import render
from django.urls import reverse_lazy

from horizon import forms

from adjutant_ui.content.signup import forms as su_forms


class SignupFormView(forms.ModalFormView):
    form_class = su_forms.SignupForm
    submit_url = reverse_lazy("horizon:signup:signup:index")
    success_url = reverse_lazy("horizon:signup:signup:submitted")
    template_name = 'signup/index.html'


def signup_sent_view(request):
    return render(request, 'signup/submitted.html')



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf import settings
from django import forms
from django import http
from django.utils.translation import gettext_lazy as _

from horizon import forms as hforms
from horizon.utils import functions as utils

from adjutant_ui.api import adjutant

# Username is ignored, use email address instead.
USERNAME_IS_EMAIL = True


class ForgotPasswordForm(hforms.SelfHandlingForm):
    username = forms.CharField(
        max_length=255, label=_("User Name"),
        widget=forms.TextInput(attrs={"autofocus": "autofocus"}))
    email = forms.EmailField(label=_("Email"))

    def __init__(self, *args, **kwargs):
        super(ForgotPasswordForm, self).__init__(*args, **kwargs)
        if (hasattr(settings, 'USERNAME_IS_EMAIL') and
                getattr(settings, 'USERNAME_IS_EMAIL')):
            self.fields.pop('username')
            self.fields['email'].widget = forms.TextInput(
                attrs={"autofocus": "autofocus"})

    def clean(self, *args, **kwargs):
        # validate username and email?
        return super(ForgotPasswordForm, self).clean(*args, **kwargs)

    def handle(self, request, data):
        try:
            submit_response = adjutant.forgotpassword_submit(
                request, data)
            if submit_response.ok:
                return True
        except Exception:
            pass

        # Send the user back to the login page.
        msg = _("The password reset service is currently unavailable. "
                "Please try again later.")
        response = http.HttpResponseRedirect(settings.LOGOUT_URL)
        utils.add_logout_reason(self.request, response, msg)
        return response



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstack_dashboard.test import helpers


class APITestCase(helpers.APITestCase):
    """Extends the base Horizon APITestCase for adjutantclient"""

    def setUp(self):
        super(APITestCase, self).setUp()



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import gettext_lazy as _

from horizon import forms
from horizon import messages

import json

from adjutant_ui.api import adjutant


class UpdateTaskForm(forms.SelfHandlingForm):
    task_id = forms.CharField(widget=forms.HiddenInput())
    task_type = forms.CharField(widget=forms.TextInput(
        attrs={'readonly': True}))
    task_data = forms.CharField(widget=forms.Textarea)

    def clean_task_data(self):
        taskdata = self.cleaned_data['task_data']
        try:
            json.loads(taskdata)
        except ValueError:
            raise forms.ValidationError(
                "Invalid non-JSON data in Task Data field")

        return taskdata

    def handle(self, request, data):
        task_id = self.cleaned_data.pop('task_id')
        try:
            response = adjutant.task_update(
                request, task_id, data['task_data'])
            if response.status_code in [200, 202]:
                messages.success(request, _('Updated task successfully.'))
            elif response.status_code == 400:
                messages.error(request, _(response.text))
            else:
                messages.error(request, _('Failed to update task.'))
            return True
        except Exception:
            messages.error(request, _('Failed to update task.'))
            return False



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf import settings
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from adjutant_ui.api import adjutant


def get_role_choices(request):
    """Get manageable roles for user.

    Returns a list of sorted 2-ary tuples containing the roles the current
    user can manage.
    """
    role_names = adjutant.valid_role_names_get(request)
    role_tuples = [(r, adjutant.get_role_text(r)) for r in role_names]
    role_tuples = sorted(role_tuples, key=lambda role: role[1])
    return role_tuples


class InviteUserForm(forms.SelfHandlingForm):
    username = forms.CharField(max_length=255, label=_("User Name"))
    email = forms.EmailField()
    roles = forms.MultipleChoiceField(label=_("Roles"),
                                      required=True,
                                      widget=forms.CheckboxSelectMultiple(),
                                      help_text=_("Select roles to grant to "
                                                  "the user within the "
                                                  "current project."))

    def __init__(self, *args, **kwargs):
        super(InviteUserForm, self).__init__(*args, **kwargs)
        if (hasattr(settings, 'USERNAME_IS_EMAIL') and
                getattr(settings, 'USERNAME_IS_EMAIL')):
            self.fields.pop('username')
        self.fields['roles'].choices = get_role_choices(self.request)
        self.fields['roles'].initial = ['_member_']

    def handle(self, request, data):
        try:
            response = adjutant.user_invite(request, data)
            if response.status_code == 202:
                messages.success(request, _('Invited user successfully.'))
            else:
                messages.error(request, _('Failed to invite user.'))
            return True
        except Exception:
            messages.error(request, _('Failed to invite user.'))
            return False


class UpdateUserForm(forms.SelfHandlingForm):
    id = forms.Field()
    id.widget = forms.HiddenInput()
    name = forms.CharField()
    name.widget.attrs['readonly'] = True

    roles = forms.MultipleChoiceField(
        label=_("Roles"),
        required=True,
        widget=forms.CheckboxSelectMultiple(),
        help_text=_("Select roles to limit the "
                    "permission of the user.")
    )

    def __init__(self, *args, **kwargs):
        super(UpdateUserForm, self).__init__(*args, **kwargs)
        self.fields['roles'].choices = get_role_choices(self.request)

    def handle(self, request, data):
        # Get the before and after role lists, make two lists:
        # roles_added and roles_removed.
        # Submit each list to the api (add first?)
        try:
            user_id = data['id']
            current_user = adjutant.user_get(request, user_id)
            current_roles = set(current_user['roles'])
            managable_roles = set(
                adjutant.valid_role_names_get(request))
            current_managable_roles = current_roles & managable_roles
            desired_roles = set(data['roles'])
            roles_added = list(desired_roles - current_managable_roles)
            roles_removed = list(current_managable_roles - desired_roles)

            # Remove roles from user
            remove_status = 202
            if len(roles_removed) > 0:
                remove_response = adjutant.user_roles_remove(
                    request,
                    user_id,
                    roles_removed)
                remove_status = remove_response.status_code
            if remove_status != 202:
                messages.error(request, _('Failed to remove roles from user.'))
                return False

            # Add new roles
            added_status = 202
            if len(roles_added) > 0:
                added_response = adjutant.user_roles_add(
                    request,
                    user_id,
                    roles_added)
                added_status = added_response.status_code
            if added_status != 202:
                messages.error(request, _('Failed to add roles to user.'))
                return False

        except Exception:
            msg = _('Failed to update user.')
            url = reverse('horizon:management:project_users:index')
            exceptions.handle(request, msg, redirect=url)
            return False

        messages.success(request, _('Updated user successfully.'))
        return True



# Copyright 2016 Catalyst IT Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import exceptions
from horizon import tables

from openstack_dashboard.dashboards.admin.defaults.tables import (
    get_compute_quota_name, get_volume_quota_name, get_network_quota_name
)

from adjutant_ui.api import adjutant


def to_caps(value):
    return value.title()


def display_as_percent(value):
    if value == "-":
        return value
    return '{:.1%}'.format(value)


def get_quota_name(quota):
    if quota.service == "nova":
        return get_compute_quota_name(quota)
    if quota.service == "cinder":
        return get_volume_quota_name(quota)
    if quota.service == "neutron":
        return get_network_quota_name(quota)

    return quota.name.replace("_", " ").title()


class UpdateQuota(tables.LinkAction):
    name = "update"
    verbose_name = _("Update Quota")
    url = "horizon:management:quota:update"
    classes = ("ajax-modal",)
    icon = "edit"


class CancelQuotaTask(tables.DeleteAction):
    help_text = _("This will cancel the selected quota update.")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Cancel Quota Update",
            u"Cancel Quota Updates",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Cancelled Quota Update",
            u"Cancelled Quota Updates",
            count
        )

    def delete(self, request, obj_id):
        result = adjutant.task_cancel(request, obj_id)
        if not result or result.status_code not in [200, 202]:
            exception = exceptions.NotAvailable()
            exception._safe_message = False
            raise exception

    def allowed(self, request, task=None):
        if task:
            return task.status == "Awaiting Approval"
        return True


class ViewRegion(tables.LinkAction):
    name = "view_region"
    verbose_name = _("View Region")
    url = "horizon:management:quota:region_detail"


class ViewSize(tables.LinkAction):
    name = "view_size"
    verbose_name = _("View Size")
    url = "horizon:management:quota:size_detail"


class UpdateQuotaRow(tables.Row):
    def load_cells(self, resource=None):
        super(UpdateQuotaRow, self).load_cells(resource)
        resource = self.datum
        if resource.important is False:
            self.attrs['hide'] = True
            self.attrs['style'] = 'display: none;'

        self.attrs['size_blob'] = json.dumps(self.datum.size_blob)


class RegionQuotaDetailTable(tables.DataTable):
    service = tables.Column("service", verbose_name=_("Service"),
                            filters=(adjutant.get_service_type, ))
    name = tables.Column(get_quota_name, verbose_name=_("Resource Name"),)
    value = tables.Column("current_quota", verbose_name=_("Resource Quota"), )
    usage = tables.Column("current_usage", verbose_name=_("Current Usage"))
    percent = tables.Column("percent", verbose_name=_("Percentage of Use"),
                            filters=(display_as_percent, ))


class QuotaDetailUsageTable(tables.DataTable):
    service = tables.Column("service", verbose_name=_("Service"),
                            filters=(adjutant.get_service_type, ))
    name = tables.Column(get_quota_name, verbose_name=_("Resource Name"),)
    value = tables.Column("value", verbose_name=_("Quota Value"), )
    current_quota = tables.Column("current_quota",
                                  verbose_name=_("Current Quota "
                                                 "(Current Region)"), )


class RegionOverviewTable(tables.DataTable):
    region = tables.Column("region", verbose_name=_("Region Name"),
                           link=("horizon:management:quota:region_detail"))
    quota_size = tables.Column("quota_size",
                               verbose_name=_("Current Quota Size"),
                               filters=(to_caps, ))
    preapproved_quotas = tables.Column(
        "preapproved_quotas", filters=(to_caps, ),
        verbose_name=_("Preapproved Quota Sizes *"))

    class Meta(object):
        name = "region_overview"
        row_actions = (UpdateQuota, ViewRegion)
        verbose_name = _("Current Quotas")
        hidden_title = False


class QuotaTasksTable(tables.DataTable):
    quota_size = tables.Column(
        "size",
        verbose_name=_("Proposed Size"),
        filters=(to_caps, ))
    regions = tables.Column("regions", verbose_name=_("For Regions"))
    user = tables.Column("user", verbose_name=_("Requested By"))
    created = tables.Column("created", verbose_name=_("Requested On"))
    valid = tables.Column("valid", verbose_name=_("Valid"))
    stats = tables.Column("status", verbose_name=_("Status"))

    class Meta(object):
        name = "quota_tasks"
        row_actions = (CancelQuotaTask, )
        verbose_name = _("Previous Quota Changes")
        hidden_title = False


class SizeOverviewTable(tables.DataTable):
    id = tables.Column("id", hidden=True)
    size = tables.Column("name", verbose_name=_("Size Name"),
                         filters=(to_caps, ))

    class Meta(object):
        name = "size_overview"
        row_actions = (ViewSize, )
        verbose_name = _("Quota Sizes")
        hidden_title = False


class ChangeSizeDisplayTable(tables.DataTable):
    service = tables.Column("service", verbose_name=_("Service"),
                            filters=(adjutant.get_service_type, ),
                            hidden=True)
    name = tables.Column(get_quota_name, verbose_name=_("Resource"),)
    current_quota = tables.Column("current_quota",
                                  verbose_name=_("Current Quota"), )
    usage = tables.Column("current_usage", verbose_name=_("Current Usage"))
    value = tables.Column("value", verbose_name=_("New Quota Value"), )

    class Meta(object):
        name = 'change_size'
        row_class = UpdateQuotaRow



# Making use of this to make a generic enabled file
# to load base adjutant-ui content.
FEATURE = "adjutant-ui-base"

# A list of applications to be added to INSTALLED_APPS.
ADD_INSTALLED_APPS = [
    'adjutant_ui',
]



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf import settings
from django.forms import ValidationError  # noqa
from django import http
from django.utils.translation import gettext_lazy as _
from django.views.decorators.debug import sensitive_variables  # noqa

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon.utils import functions as utils
from horizon.utils import validators

from openstack_dashboard import api


class PasswordForm(forms.SelfHandlingForm):
    new_password = forms.RegexField(
        label=_("New password"),
        widget=forms.PasswordInput(render_value=False),
        regex=validators.password_validator(),
        error_messages={
            'invalid': validators.password_validator_msg()
        }
    )
    confirm_password = forms.CharField(
        label=_("Confirm new password"),
        widget=forms.PasswordInput(render_value=False)
    )
    no_autocomplete = True

    def clean(self):
        '''Check to make sure password fields match.'''
        data = super(forms.Form, self).clean()
        if 'new_password' in data:
            if data['new_password'] != data.get('confirm_password', None):
                raise ValidationError(_('Passwords do not match.'))
        return data

    # We have to protect the entire "data" dict because it contains
    # newpassword strings.
    @sensitive_variables('data')
    def handle(self, request, data):
        usable_data = self.cleaned_data
        user_is_editable = api.keystone.keystone_can_edit_user()

        if user_is_editable and usable_data:
            try:
                api.keystone.user_update_own_password(
                    request,
                    usable_data['current_password'],
                    usable_data['new_password']
                )
                response = http.HttpResponseRedirect(settings.LOGOUT_URL)
                msg = _("Password changed. Please log in again to continue.")
                utils.add_logout_reason(request, response, msg)
                return response
            except Exception:
                exceptions.handle(request,
                                  _('Unable to change password.'))
                return False
        else:
            messages.error(request, _('Changing password is not supported.'))
            return False


class ConfirmForm(forms.SelfHandlingForm):

    def handle(self, request, data):
        pass



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import defaultdict

from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import exceptions
from horizon import tables

from adjutant_ui.api import adjutant


class InviteUser(tables.LinkAction):
    name = "invite"
    verbose_name = _("Invite User")
    url = "horizon:management:project_users:invite"
    classes = ("ajax-modal",)
    icon = "plus"

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Invited User",
            u"Invited Users",
            count
        )


class ResendInvitation(tables.BatchAction):
    name = "resend"

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Resend Invitation",
            u"Resend Invitations",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Invitation re-sent",
            u"Invitations re-sent",
            count
        )

    def allowed(self, request, user=None):
        # Only invited users can be re-invited
        return user.cohort == 'Invited'

    def action(self, request, datum_id):
        user = self.table.get_object_by_id(datum_id)
        adjutant.user_invitation_resend(request, user.id)


class UpdateUser(tables.LinkAction):
    name = "update"
    verbose_name = _("Update User")
    url = "horizon:management:project_users:update"
    classes = ("ajax-modal",)
    icon = "pencil"

    def allowed(self, request, user=None):
        # Currently, we only support updating existing users
        return user.cohort == 'Member'


class RevokeUser(tables.DeleteAction):
    help_text = _("This will remove the selected user(s) from the current "
                  "project.")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Revoke User",
            u"Revoke Users",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Revoked User",
            u"Revoked Users",
            count
        )

    def delete(self, request, obj_id):
        user = self.table.get_object_by_id(obj_id)

        # NOTE(dale): There is a different endpoint to revoke an invited user
        #             than an Active user.
        result = None
        if user.cohort == 'Invited':
            # Revoke invite for the user
            result = adjutant.user_revoke(request, user.id)
        else:
            # Revoke all roles from the user.
            # That'll get them out of our project; they keep their account.
            result = adjutant.user_roles_remove(
                request, user.id, user.roles)
        if not result or result.status_code not in [200, 202]:
            exception = exceptions.NotAvailable()
            exception._safe_message = False
            raise exception


class UpdateUserRow(tables.Row):
    ajax = True

    def get_data(self, request, user_id):
        user = adjutant.user_get(request, user_id)
        return user

    def load_cells(self, user=None):
        super(UpdateUserRow, self).load_cells(user)
        user = self.datum
        self.classes.append('category-' + user.cohort)


class CohortFilter(tables.FixedFilterAction):
    def get_fixed_buttons(self):
        def make_dict(text, value, icon):
            return dict(text=text, value=value, icon=icon)

        buttons = []
        buttons.append(make_dict(_('Project Users'),
                                 'Member',
                                 'fa-user-md'))
        buttons.append(make_dict(_('Invited Users'),
                                 'Invited',
                                 'fa-user'))
        return buttons

    def categorize(self, table, users):
        categorized_users = defaultdict(list)
        for u in users:
            categorized_users[u.cohort].append(u)
        return categorized_users


def UserRoleDisplayFilter(role_list):
    roles = [adjutant.get_role_text(r) for r in role_list]
    return ', '.join(roles)


class UsersTable(tables.DataTable):
    uid = tables.Column('id', verbose_name=_('User ID'))
    name = tables.Column('name', verbose_name=_('Name'))
    email = tables.Column('email', verbose_name=_('Email'))
    roles = tables.Column('roles',
                          verbose_name=_('Roles'),
                          filters=[UserRoleDisplayFilter])
    status = tables.Column('status', verbose_name=_('Status'))
    cohort = tables.Column('cohort',
                           verbose_name=_('Member Type'),
                           hidden=True)

    class Meta(object):
        name = 'users'
        row_class = UpdateUserRow
        verbose_name = _('Users')
        columns = ('id', 'name', 'email', 'roles', 'status', 'cohort')
        table_actions = (CohortFilter, InviteUser, RevokeUser)
        row_actions = (UpdateUser, ResendInvitation, RevokeUser)
        multi_select = True



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs

from adjutant_ui.api import adjutant
from adjutant_ui.content.tasks import tables as task_tables


class ActiveTaskListTab(tabs.TableTab):
    table_classes = (task_tables.TaskTable,)
    template_name = 'horizon/common/_detail_table.html'
    page_title = _("Active Tasks")
    name = _('Active')
    slug = 'active'
    filters = {'cancelled': {'exact': False},
               'approved': {'exact': False}}
    _prev = False
    _more = False

    def get_task_table_data(self):
        tasks = []
        marker = self.request.GET.get(
            self.table_classes[0]._meta.pagination_param, 1)
        try:
            tasks, self._prev, self._more = adjutant.task_list(
                self.request, filters=self.filters, page=marker)
        except Exception:
            exceptions.handle(self.request, _('Failed to list tasks.'))
        return tasks

    def has_prev_data(self, table):
        return self._prev

    def has_more_data(self, table):
        return self._more


class ApprovedTaskListTab(ActiveTaskListTab):
    table_classes = (task_tables.ApprovedTaskTable,)
    page_title = _("Approved Tasks")
    name = _('Approved')
    slug = 'approved'
    filters = {'cancelled': {'exact': False},
               'approved': {'exact': True},
               'completed': {'exact': False}}

    def get_approved_table_data(self):
        return super(ApprovedTaskListTab, self).get_task_table_data()


class CompletedTaskListTab(ActiveTaskListTab):
    table_classes = (task_tables.CompletedTaskTable,)
    page_title = _("Completed Tasks")
    name = _('Completed')
    slug = 'completed'
    filters = {'completed': {'exact': True}}

    def get_completed_table_data(self):
        return super(CompletedTaskListTab, self).get_task_table_data()


class CancelledTaskListTab(ActiveTaskListTab):
    table_classes = (task_tables.CancelledTaskTable,)
    name = _('Cancelled')
    slug = 'cancelled'
    filters = {'cancelled': {'exact': True}}

    def get_cancelled_table_data(self):
        return super(CancelledTaskListTab, self).get_task_table_data()


class TaskTabs(tabs.TabGroup):
    slug = "tasks"
    tabs = (ActiveTaskListTab, ApprovedTaskListTab, CompletedTaskListTab,
            CancelledTaskListTab)
    sticky = True

    def get_selected_tab(self):
        super(TaskTabs, self).get_selected_tab()
        if not self._selected:
            for tab in self.tabs:
                param = tab.table_classes[0]._meta.pagination_param
                if self.request.GET.get(param):
                    self._selected = self.get_tab(tab.slug)
                    return self._selected


class TaskOverviewTab(tabs.Tab):
    name = _("Overview")
    slug = "overview"
    template_name = 'management/tasks/_task_detail_overview.html'

    def get_context_data(self, request):
        return {"task": self.tab_group.kwargs['task']}


class TaskActionsTab(tabs.Tab):
    name = _("Actions")
    slug = "actions"
    template_name = 'management/tasks/_task_detail_actions.html'

    def get_context_data(self, request):
        return {"task": self.tab_group.kwargs['task']}


class TaskNotesTab(tabs.Tab):
    name = _("Action Notes")
    slug = "notes"
    template_name = 'management/tasks/_task_detail_notes.html'

    def get_context_data(self, request):
        return {"task": self.tab_group.kwargs['task']}


class TaskDetailTabs(tabs.DetailTabsGroup):
    slug = "task_details"
    tabs = (TaskOverviewTab, TaskActionsTab, TaskNotesTab)



#
# Copyright 2016 Catalyst IT Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.conf import settings
from django import template
from django.utils.translation import gettext_lazy

register = template.Library()


@register.simple_tag(takes_context=True)
def relabel_username_field(context):
    """Takes the username field and relabels it to 'email'.

    Note(dalees):
    This function modifies context inside a template renderer,
    which is really bad practice MVC. In this case we prefer
    not to modify the openstack_auth form module, so changing
    form label directly is our best option. Avoid if you can!
    """

    if (hasattr(settings, 'USERNAME_IS_EMAIL') and
            getattr(settings, 'USERNAME_IS_EMAIL')):
        try:
            context['form'].fields['username'].label = gettext_lazy('Email')
        except Exception:
            pass
    return u""



# Copyright 2013 Centrin Data Systems Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.forms import ValidationError
from django.utils.translation import gettext_lazy as _

from horizon import forms
from horizon import messages

from adjutant_ui import api


class EmailForm(forms.SelfHandlingForm):
    new_email = forms.EmailField(
        label=_("New email address"),
        required=True)

    confirm_email = forms.CharField(
        label=_("Confirm email address"),
        required=True)
    no_autocomplete = True

    def clean(self):
        '''Check to make sure email fields match.'''
        data = super(forms.Form, self).clean()
        if data.get('new_email') != data.get('confirm_email', None):
            raise ValidationError(_('Email addresses do not match.'))
        return data

    def handle(self, request, data):
        try:
            response = api.adjutant.email_update(request, data['new_email'])
            if response.status_code == 202:
                msg = _("Confirmation email sent to %s.")
                messages.success(request, msg % data['new_email'])
            elif response.status_code == 400:
                messages.warning(request, _(
                    'Unable to update email. May already be in use.'))
            else:
                messages.error(request, _('Failed to update email.'))
            return True
        except Exception as e:
            messages.error(request, _('Failed to update email. %s' % str(e)))
            return False



# The slug of the panel to be added to HORIZON_CONFIG. Required.
PANEL = 'email'
# The slug of the dashboard the PANEL associated with. Required.
PANEL_DASHBOARD = 'settings'
# The slug of the panel group the PANEL is associated with.
PANEL_GROUP = 'default'

# Python panel class of the PANEL to be added.
ADD_PANEL = 'adjutant_ui.content.email.panel.EmailPanel'



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import exceptions
from horizon import tables

from adjutant_ui.api import adjutant


class CancelTask(tables.DeleteAction):
    help_text = _("This will cancel all selected tasks.")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Cancel Task",
            u"Cancel Tasks",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Cancelled Task",
            u"Cancelled Tasks",
            count
        )

    def delete(self, request, obj_id):
        result = adjutant.task_cancel(request, obj_id)
        if not result or result.status_code not in [200, 202]:
            exception = exceptions.NotAvailable()
            exception._safe_message = False
            raise exception

    def allowed(self, request, task=None):
        if task:
            return not(
                task.status == "Completed" or task.status == "Cancelled")
        return True


class ApproveTask(tables.BatchAction):
    name = "approve"
    help_text = _("This will approve all of the selected tasks.")
    action_type = "danger"

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Approve Task",
            u"Approve Tasks",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Approved Task",
            u"Approved Tasks",
            count
        )

    def action(self, request, obj_id):
        result = adjutant.task_approve(request, obj_id)
        if not result or result.status_code not in [200, 202]:
            exception = exceptions.NotAvailable()
            exception._safe_message = False
            raise exception

    def allowed(self, request, task=None):
        if task:
            return task.valid and not(
                task.status == "Completed" or task.status == "Cancelled")
        return True


class ReissueToken(tables.BatchAction):
    name = "reissue"
    help_text = _("This will reissue tokens for the selected tasks.")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Reissue Token",
            u"Reissue Tokens",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Reissued Token",
            u"Reissued Tokens",
            count
        )

    def action(self, request, obj_id):
        result = adjutant.token_reissue(request, obj_id)
        if not result or result.status_code not in [200, 202]:
            exception = exceptions.NotAvailable()
            exception._safe_message = False
            raise exception

    def allowed(self, request, task=None):
        if task:
            return task.status == "Approved; Incomplete"
        return True


class RevalidateTask(tables.BatchAction):
    name = "revalidate"
    help_text = _("Rerun initial validation for the task.")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Rerun Validation",
            u"Rerun Validation",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Validation run",
            u"Validation run",
            count
        )

    def action(self, request, obj_id):
        result = adjutant.task_revalidate(request, obj_id)
        if not result or result.status_code not in [200, 202]:
            exception = exceptions.NotAvailable()
            exception._safe_message = False
            raise exception

    def allowed(self, request, task=None):
        if task:
            return task.status == 'Awaiting Approval' and not task.valid
        return True


class ReapproveTask(ApproveTask):
    name = "approve"
    help_text = _("This will approve all of the selected tasks.")
    action_type = "danger"

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Reapprove Task",
            u"Reapprove Tasks",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Reapproved Task",
            u"Reapproved Tasks",
            count
        )


class UpdateTask(tables.LinkAction):
    name = "update"
    verbose_name = _("Update Task")
    url = "horizon:management:tasks:update"
    classes = ("ajax-modal",)

    def allowed(self, request, task=None):
        if task:
            return task.status == 'Awaiting Approval'
        return True


def TaskTypeDisplayFilter(task_type):
    return task_type.replace("_", " ").title()


class TaskTable(tables.DataTable):
    uuid = tables.Column('id', verbose_name=_('Task ID'),
                         hidden=True)
    task_type = tables.Column('task_type', verbose_name=_('Task Type'),
                              filters=[TaskTypeDisplayFilter],
                              link="horizon:management:tasks:detail")
    status = tables.Column('status', verbose_name=_('Status'))
    request_by = tables.Column('request_by', verbose_name=_('Requestee'))
    request_project = tables.Column('request_project',
                                    verbose_name=_('Request Project'))
    valid = tables.Column('valid', verbose_name=_("Actions Valid"))
    created_on = tables.Column('created_on',
                               verbose_name=_('Request Date'))
    page = tables.Column('page', hidden=True)

    class Meta(object):
        name = 'task_table'
        verbose_name = _('Tasks')
        table_actions = (ApproveTask, RevalidateTask, CancelTask)
        row_actions = (ApproveTask, UpdateTask, RevalidateTask, CancelTask, )
        prev_pagination_param = pagination_param = 'task_page'

    def get_prev_marker(self):
        return str(int(self.data[0].page) - 1) if self.data else ''

    def get_marker(self):
        return str(int(self.data[0].page) + 1) if self.data else ''

    def get_object_display(self, obj):
        task_type = obj.task_type.replace("_", " ").title()
        return "%s (%s)" % (task_type, obj.id)


class ApprovedTaskTable(TaskTable):

    class Meta(object):
        name = 'approved_table'
        verbose_name = _('Tasks')
        table_actions = (CancelTask, ReapproveTask, ReissueToken)
        row_actions = (CancelTask, ReapproveTask, ReissueToken)
        prev_pagination_param = pagination_param = 'approved_page'


class CompletedTaskTable(TaskTable):

    class Meta(object):
        name = 'completed_table'
        verbose_name = _('Tasks')
        prev_pagination_param = pagination_param = 'completed_page'


class CancelledTaskTable(TaskTable):

    class Meta(object):
        name = 'cancelled_table'
        verbose_name = _('Tasks')
        prev_pagination_param = pagination_param = 'cancelled_page'



# The name of the dashboard to be added to HORIZON['dashboards']. Required.
DASHBOARD = 'token'

# A list of applications to be added to INSTALLED_APPS.
ADD_INSTALLED_APPS = [
    'adjutant_ui.dashboards.token_dash',
    'adjutant_ui.content.token',
]



from django import template
import json

register = template.Library()


@register.simple_tag
def pretty_json(value):
    return json.dumps(value, indent=4)


@register.simple_tag
def pretty_list(value):
    return ', '.join(value)


register.filter('pretty_json', pretty_json)
register.filter('pretty_list', pretty_list)



# Copyright 2016 Catalyst IT Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.urls import reverse  # noqa
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from adjutant_ui.api import adjutant


LOG = logging.getLogger(__name__)


class UpdateQuotaForm(forms.SelfHandlingForm):
    region = forms.CharField(label=_("Region"))
    region.widget.attrs['readonly'] = True
    size = forms.ChoiceField(label=_("Size"))
    size.widget.attrs['onchange'] = 'updateSizeTable()'

    failure_url = 'horizon:management:quota:index'
    submit_url = 'horizon:management:quota:update'
    success_url = "horizon:management:quota:index"

    def __init__(self, *args, **kwargs):
        size_choices = kwargs.pop('size_choices')
        super(UpdateQuotaForm, self).__init__(*args, **kwargs)
        self.fields['size'].choices = size_choices

    def handle(self, request, data):
        try:
            response = adjutant.update_quotas(request, data['size'],
                                              regions=[data['region']])
            if response.status_code == 202:
                messages.success(request, _('Task created and may require '
                                            'admin approval.'))
            elif response.status_code == 400:
                messages.error(request, _('Failed to update quota. You may'
                                          ' have usage over the new values '
                                          'that you are attempting to update'
                                          ' the quota to.'))
            else:
                messages.error(request, _('Failed to update quota.'))
            return True
        except Exception:
            msg = _('Failed to update quota.')
            url = reverse('horizon:management:quota:index')
            exceptions.handle(request, msg, redirect=url)
            return False



# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs

from adjutant_ui.api import adjutant
from adjutant_ui.api.adjutant import AdjutantApiError
from adjutant_ui.content.notifications import tables as notification_tables


class UnacknowledgedNotificationsTab(tabs.TableTab):
    table_classes = (notification_tables.NotificationTable,)
    template_name = 'horizon/common/_detail_table.html'
    page_title = _("Unacknowledged Notifications")
    name = _('Unacknowledged')
    slug = 'unacknowledged'
    filters = {'acknowledged': {'exact': False}}
    _prev = False
    _more = False
    _page = 1

    def get_notification_table_data(self):
        notifications = []
        self._page = self.request.GET.get(
            self.table_classes[0]._meta.pagination_param, 1)
        try:
            notifications, self._prev, self._more = adjutant.notification_list(
                self.request, filters=self.filters, page=self._page)
        except AdjutantApiError as e:
            if e.message != "Empty Page":
                raise
            try:
                self._page = 1
                notifications, self._prev, self._more = \
                    adjutant.notification_list(
                        self.request, filters=self.filters,
                        page=self._page)
            except Exception:
                exceptions.handle(
                    self.request, _('Failed to list notifications.'))
        except Exception:
            exceptions.handle(self.request, _('Failed to list notifications.'))
        return notifications

    def has_prev_data(self, table):
        table.page = self._page
        return self._prev

    def has_more_data(self, table):
        table.page = self._page
        return self._more


class AcknowlededNotificationsTab(UnacknowledgedNotificationsTab):
    table_classes = (notification_tables.AcknowlegedNotificationTable,)
    page_title = _("Acknowleged Notifications")
    name = _('Acknowleged')
    slug = 'acknowledged'
    filters = {'acknowledged': {'exact': True}}

    def get_acknowleged_table_data(self):
        notifications = []
        self._page = self.request.GET.get(
            self.table_classes[0]._meta.pagination_param, 1)
        try:
            notifications, self._prev, self._more = adjutant.notification_list(
                self.request, filters=self.filters, page=self._page)
        except Exception:
            exceptions.handle(self.request, _('Failed to list notifications.'))
        return notifications


class NotificationTabGroup(tabs.TabGroup):
    slug = "notifications"
    tabs = (UnacknowledgedNotificationsTab, AcknowlededNotificationsTab, )
    sticky = True

    def get_selected_tab(self):
        super(NotificationTabGroup, self).get_selected_tab()
        if not self._selected:
            for tab in self.tabs:
                param = tab.table_classes[0]._meta.pagination_param
                if self.request.GET.get(param):
                    self._selected = self.get_tab(tab.slug)
                    return self._selected
